@@ -7,7 +7,7 @@
 # Need use lib here because CGI scripts don't have access to
 # the PerlSwitches used in httpd.conf.
 
-use lib '/home/ron/perl.modules/CGI-Up/lib';
+use lib '/home/ron/perl.modules/CGI-Uploader/lib';
 use strict;
 use warnings;
 
